<?php

/*	
	Version: 1.0.0b2
	Released 7/21/2003
	This is a generic PHP to show how to interface to the InterShipper server.
	The data returned from the IS server is XML format
	This script does the work in three phases:
		1) via fsockopen sends the shipping information via GET
			to the Intershipper server with the long list of parameters ($uri).
		2) the returned XML data is cleaned up and sent to the XML parser to be
			packaged into a more "usable" data structure.
		
		3) the more "usable" data structure is rolled out into a drop-down menu
			that takes the form of one used for a checkout process on 
			an ecommerce site.
*/

//Set your username and password.
$username = 'doug';
$password = 'password';

// Build the query string to be sent to the IS server.
//http://intershipper.com/Shipping/Intershipper/Website/MainPage.jsp?Page=Integrate
// for additional information

$url = 'www.intershipper.com';
$uri = '/Interface/Intershipper/XML/v2.0/HTTP.jsp?'.
	'Username=' . $username . 
	'&Password=' . $password . 
	'&Version=' . '2.0.0.0' .
	'&ShipmentID=' . '1234' . 
	'&QueryID=' . '23456' . 
	'&TotalCarriers=' . '1' . 
	'&CarrierCode1=' . 'UPS' . 
	'&CarrierAccount1=' . '123456789' . 
	'&CarrierInvoiced1=' . '1' . 
	'&TotalClasses=' . '1' . 
	'&ClassCode1=' . '1DY' . 
	'&DeliveryType=' . 'COM' . 
	'&ShipMethod=' . 'DRP' . 
	'&OriginationName=' . 'John%20Smith' . 
	'&OriginationAddress1=' . '465%20Tyende%20Ovi' . 
	'&OriginationCity=' . 'Flagstaff' . 
	'&OriginationState=' . 'AZ' . 
	'&OriginationPostal=' . '86001' . 
	'&OriginationCountry=' . 'US' . 
	'&DestinationName=' . 'Jane%20Doe' . 
	'&DestinationAddress1=' . '466%20Tyende%20Ovi' . 
	'&DestinationCity=' . 'Flagstaff' . 
	'&DestinationState=' . 'AZ' . 
	'&DestinationPostal=' . '86001' . 
	'&DestinationCountry=' . 'US' . 
	'&Currency=' . 'USD' . 
	'&TotalPackages=' . '1' . 
	'&BoxID1=' . '1' . 
	'&Weight1=' . '100' . 
	'&WeightUnit1=' . 'LB' . 
	'&Length1=' . '10' . 
	'&Width1=' . '10' . 
	'&Height1=' . '10' . 
	'&DimensionalUnit1=' . 'IN' . 
	'&Packaging1=' . 'BOX' . 
	'&Contents1=' . 'OTR' . 
	'&Cod1=' . '18500' . 
	'&Insurance1=' . '15000' . 
	'&TotalOptions=' . '1' . 
	'&OptionCode1=' . 'SDD';

//Define some global vars for later use
$state = array();
global $state;
$quote = array();
global $quote;
$quotes = array();
global $quotes;
global $package_id;
global $boxID;

// funtion to handle the start elements for the XML data
function startElement(&$Parser, &$Elem, $Attr) {
	global $state;
	array_push ($state, $Elem);
	$states = join (' ',$state);
	//check what state we are in
	if ($states == "SHIPMENT PACKAGE") {
		global $package_id;
		$package_id = $Attr['ID'];
	}
	//check what state we are in 
	elseif ($states == "SHIPMENT PACKAGE QUOTE") {
		global $package_id;
		global $quote;
		$quote = array ( package_id => $package_id, id => $Attr['ID']);
	}
}

//funtion to parse the XML data. The routine does a series of conditional
//checks on the data to determine where in the XML stack "we" are.
//
function characterData($Parser, $Line) {  			
	global $state;
	$states = join (' ',$state);	
	if ($states == "SHIPPMENT ERROR") {
		$error = $Line;
	} 
	elseif ($states == "SHIPMENT PACKAGE BOXID") {
		global $boxID;
		$boxID = $Line;
	} 
  	elseif ($states == "SHIPMENT PACKAGE QUOTE CARRIER NAME") {
		global $quote;
		$quote{carrier_name} = $Line;
	} 
	elseif ($states == "SHIPMENT PACKAGE QUOTE CARRIER CODE") {
		global $quote;
		$quote{carrier_code} = $Line;
	} 
	elseif ($states == "SHIPMENT PACKAGE QUOTE CLASS NAME") {
		global $quote;
		$quote{class_name} = $Line;
	} 
	elseif ($states == "SHIPMENT PACKAGE QUOTE CLASS CODE") {
		global $quote;
		$quote{class_code} = $Line;
	} 
	elseif ($states == "SHIPMENT PACKAGE QUOTE SERVICE NAME") {
		global $quote;
		$quote{service_name} = $Line;
	} 
	elseif ($states == "SHIPMENT PACKAGE QUOTE SERVICE CODE") {
		global $quote;
		$quote{service_code} = $Line;
	} 
	elseif ($states == "SHIPMENT PACKAGE QUOTE RATE AMOUNT") {
		global $quote;
		$quote['amount'] = $Line;
	}
}

// this function handles the end elements.
// once encountered it sticks the quote into the hash $quotes
// for easy access later
function endElement($Parser, $Elem) {
	global $state;	
	$states = join (' ',$state);	
	if ($states == "SHIPMENT PACKAGE QUOTE") {
		global $quote;
		global $boxID;
		global $quotes;
		unset ($quote['id']);
		unset ($quote['package_id']);
		// the $key is a combo of the carrier_code and service_code
		// this is the logical way to key each quote returned 
		$key = $quote['carrier_code'] . ' ' . $quote['service_code'];
		$quotes[$boxID][$key] = $quote;
	}
	array_pop($state);
}

//Start printing the demo HTML page
echo "<html><body><pre><form><select>\n";
//Send the socket request with the uri/url
$fp = fsockopen ($url, 80, $errno, $errstr, 30);
if (!$fp) {
    echo "Error: $errstr ($errno)<br>\n";
} 
else {
	$depth = array();
	fputs ($fp, "GET $uri HTTP/1.0\r\nHost: $url\r\n\r\n");
	//define the XML parsing routines/functions to call
	//based on the handler state
	$xml_parser = xml_parser_create();
	xml_set_element_handler($xml_parser, "startElement", "endElement");
	xml_set_character_data_handler($xml_parser, "characterData");	
	//now lets roll through the data
	while ($data = fread($fp, 4096)) {
 		$newdata = $data;
 		/*fsockopen returns more infomation than we'd like. here we 
 			remove the excess data.
 		*/
 		$newdata = preg_replace('/\r\n\r\n/', "", $newdata);
 		$newdata = preg_replace('/^HTTP.*\r\n/', "", $newdata);
 		$newdata = preg_replace('/^Date.*\r\n/', "", $newdata);
 		$newdata = preg_replace('/^Server.*\r\n/', "", $newdata);
 		$newdata = preg_replace('/Con.*/', "", $newdata);
 		$newdata = preg_replace('/Set.*/', "", $newdata);
 		$newdata = preg_replace('/\r/', "", $newdata);
 		$newdata = preg_replace('/\n/', "", $newdata);
		
		/* if we properl cleaned up the XML stream/data we can now hand it off 
		to an XML parser without error
		*/
		if (!xml_parse($xml_parser, $newdata, feof($fp))) {
    	    die(sprintf("XML error: %s at line %d",
			xml_error_string(xml_get_error_code($xml_parser)),
			xml_get_current_line_number($xml_parser)));
		}
	}
	//clean up the parser object
	xml_parser_free($xml_parser);
}


/*
Here we build a drop down menu list (as an example).
print_r $quotes
can help you debug or use the $quotes hash we built above.
a variety of info is included but mostly we probably want amount, carrier_name,
service_name.

*/
while(list($quotedata, $boxID)=each($quotes)){
	while(list($key, $bar)=each($boxID)){
		$boxID[$key]['amount'] = $boxID[$key]['amount'] / 100;
		$boxID[$key]['amount']= number_format($boxID[$key]['amount'], 2, '.', ' ');
		$boxID[$key]['amount'] = '$'.$boxID[$key]['amount'];	
		print "\n<option>";
		print $key . " ";
		print $boxID[$key]['carrier_name']." ";
		print $boxID[$key]['service_name']." ";
		print $boxID[$key]['amount'];
	}
}
echo "\n</select></form>";



/*
We're done!  

Here is an example of the data returned from the XML parser

SHIPMENT:   
  VERSION: 2.0.0.0
  SHIPMENTID: 1234
  QUERYID: 23456
  DELIVERYTYPE: COM
  SHIPMETHOD: DRP
  ORIGINATION:     
    NAME: John Smith
    ADDRESS1: 465 Tyende Ovi
    ADDRESS2:     
    ADDRESS3:     
    CITY: Flagstaff
    STATE: AZ
    POSTAL: 86001
    COUNTRY: US
  DESTINATION:     
    NAME: Jane Doe
    ADDRESS1: 466 Tyende Ovi
    ADDRESS2:     
    ADDRESS3:     
    CITY: Flagstaff
    STATE: AZ
    POSTAL: 86001
    COUNTRY: US
  PACKAGE:     
    BOXID: 1
    WEIGHT: 10.0
    DIMENSIONS:       
      LENGTH: 10.0
      WIDTH: 10.0
      HEIGHT: 10.0
    PACKAGING: BOX
    CONTENTS: OTR
    COD: 18500
    INSURANCE: 15000
    OPTIONS:       
      SPECIALHANDLING:         
      CODE: SDD
        NAME: Saturday Delivery
    QUOTE:       
      CARRIER:         
        CODE: UPS
        NAME: United Parcel Service
        ACCOUNT: 123456789
        INVOICED: 1
      CLASS:         
        CODE: 1DY
        NAME: 1st Day
      SERVICE:         
        CODE: NDA
        NAME: Next Day Air
      RATE:         
        AMOUNT: 4201
        CURRENCY: USD
    QUOTE:       
      CARRIER:         
        CODE: UPS
        NAME: United Parcel Service
        ACCOUNT: 123456789
        INVOICED: 1
      CLASS:         
        CODE: 1DY
        NAME: 1st Day
      SERVICE:         
        CODE: NDS
        NAME: Next Day Air Saver
      RATE:         
        AMOUNT: 3898
        CURRENCY: USD
    QUOTE:       
      CARRIER:         
        CODE: UPS
        NAME: United Parcel Service
        ACCOUNT: 123456789
        INVOICED: 1
      CLASS:         
        CODE: 1DY
        NAME: 1st Day
      SERVICE:         
        CODE: NDM
        NAME: Next Day Air AM
      RATE:         
        AMOUNT: 7087
        CURRENCY: USD
*/
?>